/**************************************************************************
 *
 *	project				 : qtplot
 *
 *	copyright            : (C) 2004 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source: $
 *
 *  $Revision: $
 *
 *	$Log: $
 *	
 *
 ***************************************************************************/
/***************************************************************************
 *																		   *
 * This file is part of the qtplot package (a gnuplot plotting frontend    *
 * for Qtopia and Qt)                                                      *
 *																		   *
 * Copyright (C) 2004 by Michael Neuroth.								   *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation; either version 2 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 ***************************************************************************/
/*

  Work for Version 0.2:

  15.5.2004		added some more linetypes and point types,
				Bugfix: texts can contain spaces
				Bugfix: better y alignment of texts
  16.5.2004		scaling of text and labels improved
				Printing implemented


 */
#include "graphicoperation.h"

#include <qcanvas.h>

// ************************************************************************

GraphicEnv::GraphicEnv( QCanvas * pCanvas )
: m_pCanvas( pCanvas ),
  m_x( 0 ),
  m_y( 0 )
{
}

void GraphicEnv::ClearCanvas()
{
	m_pCanvas = 0;
}

bool GraphicEnv::IsCanvasOk() const
{
	return m_pCanvas != 0;
}

void GraphicEnv::SetBrush( const QBrush & aBrush )
{
	m_aActBrush = aBrush;
}

QBrush GraphicEnv::GetBrush() const
{
	return m_aActBrush;
}

void GraphicEnv::SetPen( const QPen & aPen )
{
	m_aActPen = aPen;
}

QPen GraphicEnv::GetPen() const
{
	return m_aActPen;
}

void GraphicEnv::SetPos( int x, int y )
{
	m_x = x;
	m_y = y;
}

void GraphicEnv::GetPos( int & x, int & y ) const
{
	x = m_x;
	y = m_y;
}

int	GraphicEnv::ScaleX( int x ) const
{
	return (QTOPIA_X_OFFSET+x)*m_pCanvas->width()/MAX_QTOPIA_WIDTH;
}

int	GraphicEnv::ScaleY( int y ) const
{
	return (MAX_QTOPIA_HEIGHT-y-QTOPIA_Y_OFFSET)*m_pCanvas->height()/MAX_QTOPIA_HEIGHT;
}

int GraphicEnv::RelScaleX( int x ) const
{
	return x*m_pCanvas->width()/MAX_QTOPIA_WIDTH;
}

int GraphicEnv::RelScaleY( int y ) const
{
	return y*m_pCanvas->height()/MAX_QTOPIA_HEIGHT;
}

QCanvas * GraphicEnv::GetCanvas() const
{
	return m_pCanvas;
}

// ************************************************************************

GraphicOperation::~GraphicOperation()
{
}

void GraphicOperation::Draw( GraphicEnv & env )
{
	// dummy implementation
}

void GraphicOperation::Clear( GraphicEnv & env )
{
	// dummy implementation
}

// ************************************************************************

GrOpMove::GrOpMove( int x, int y )
{
	m_x = x;
	m_y = y;
}

int GrOpMove::GetX( GraphicEnv & env ) const
{
	return env.ScaleX( m_x );
}

int GrOpMove::GetY( GraphicEnv & env ) const
{
	return env.ScaleY( m_y );
}

void GrOpMove::Draw( GraphicEnv & env )
{
	env.SetPos( m_x, m_y );
}

// ************************************************************************

GrOpPoint::GrOpPoint( int x, int y, int iPoint )
: GrOpMove( x, y ),
  m_pPoint1( 0 ),
  m_pPoint2( 0 ),
  m_pPoint3( 0 ),
  m_pPoint4( 0 )
{
	m_iPoint = iPoint;
}

GrOpPoint::~GrOpPoint()
{
	delete m_pPoint1;
	delete m_pPoint2;
	delete m_pPoint3;
	delete m_pPoint4;
}

void GrOpPoint::Draw( GraphicEnv & env )
{
	if( !m_pPoint1 )
	{
		switch( m_iPoint )
		{
			case 0:
			{
				QCanvasEllipse * pPoint = new QCanvasEllipse( env.GetCanvas() );
				pPoint->setSize( env.RelScaleX( POINT_X ), env.RelScaleY( POINT_Y ) );
				pPoint->setPen( env.GetPen() );
				pPoint->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint;
			}	break;
			case 1:
			{
				QCanvasLine * pPoint1 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint2 = new QCanvasLine( env.GetCanvas() );
				pPoint1->setPoints( -5,0,5,0 );
				pPoint1->setPen( env.GetPen() );
				pPoint1->setBrush( env.GetPen().color() );
				pPoint2->setPoints( 0,-5,0,5 );
				pPoint2->setPen( env.GetPen() );
				pPoint2->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint1;
				m_pPoint2 = pPoint2;
			}	break;
			case 2:
			{
				QCanvasLine * pPoint1 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint2 = new QCanvasLine( env.GetCanvas() );
				pPoint1->setPoints( -5,5,5,-5 );
				pPoint1->setPen( env.GetPen() );
				pPoint1->setBrush( env.GetPen().color() );
				pPoint2->setPoints( -5,-5,5,5 );
				pPoint2->setPen( env.GetPen() );
				pPoint2->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint1;
				m_pPoint2 = pPoint2;
			}	break;
			case 3:
			{
				QCanvasLine * pPoint1 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint2 = new QCanvasLine( env.GetCanvas() );
				pPoint1->setPoints( -5,5,5,-5 );
				pPoint1->setPen( env.GetPen() );
				pPoint1->setBrush( env.GetPen().color() );
				pPoint2->setPoints( -5,-5,5,5 );
				pPoint2->setPen( env.GetPen() );
				pPoint2->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint1;
				m_pPoint2 = pPoint2;

				QCanvasLine * pPoint3 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint4 = new QCanvasLine( env.GetCanvas() );
				pPoint3->setPoints( -5,0,5,0 );
				pPoint3->setPen( env.GetPen() );
				pPoint3->setBrush( env.GetPen().color() );
				pPoint4->setPoints( 0,-5,0,5 );
				pPoint4->setPen( env.GetPen() );
				pPoint4->setBrush( env.GetPen().color() );
				m_pPoint3 = pPoint3;
				m_pPoint4 = pPoint4;
			}	break;
			case 4:
			{
				QCanvasLine * pPoint1 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint2 = new QCanvasLine( env.GetCanvas() );
				pPoint1->setPoints( -5,5,5,5 );
				pPoint1->setPen( env.GetPen() );
				pPoint1->setBrush( env.GetPen().color() );
				pPoint2->setPoints( -5,-5,5,-5 );
				pPoint2->setPen( env.GetPen() );
				pPoint2->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint1;
				m_pPoint2 = pPoint2;

				QCanvasLine * pPoint3 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint4 = new QCanvasLine( env.GetCanvas() );
				pPoint3->setPoints( -5,-5,-5,5 );
				pPoint3->setPen( env.GetPen() );
				pPoint3->setBrush( env.GetPen().color() );
				pPoint4->setPoints( 5,-5,5,5 );
				pPoint4->setPen( env.GetPen() );
				pPoint4->setBrush( env.GetPen().color() );
				m_pPoint3 = pPoint3;
				m_pPoint4 = pPoint4;
			}	break;
			case 5:
			{
				QCanvasLine * pPoint1 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint2 = new QCanvasLine( env.GetCanvas() );
				pPoint1->setPoints( -5,0,0,5 );
				pPoint1->setPen( env.GetPen() );
				pPoint1->setBrush( env.GetPen().color() );
				pPoint2->setPoints( 0,5,5,0 );
				pPoint2->setPen( env.GetPen() );
				pPoint2->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint1;
				m_pPoint2 = pPoint2;

				QCanvasLine * pPoint3 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint4 = new QCanvasLine( env.GetCanvas() );
				pPoint3->setPoints( -5,0,0,-5 );
				pPoint3->setPen( env.GetPen() );
				pPoint3->setBrush( env.GetPen().color() );
				pPoint4->setPoints( 0,-5,5,0 );
				pPoint4->setPen( env.GetPen() );
				pPoint4->setBrush( env.GetPen().color() );
				m_pPoint3 = pPoint3;
				m_pPoint4 = pPoint4;
				/*
				QCanvasRectangle * pPoint = new QCanvasRectangle( env.GetCanvas() );
				pPoint->setSize( 10, 10 );
				pPoint->setX( 10 );
				pPoint->setY( 10 );
				pPoint->setPen( env.GetPen() );
				pPoint->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint;
				*/
			}	break;
			case 6:
			{
				QCanvasEllipse * pPoint = new QCanvasEllipse( env.GetCanvas() );
				pPoint->setSize( 10, 10 );
				pPoint->setPen( env.GetPen() );
				pPoint->setBrush( QBrush( env.GetPen().color(), Qt::NoBrush ) );
				m_pPoint1 = pPoint;
			}	break;
			case 7:
			{
				QCanvasEllipse * pPoint = new QCanvasEllipse( env.GetCanvas() );
				pPoint->setSize( 10, 10 );
				pPoint->setPen( env.GetPen() );
				pPoint->setBrush( QBrush( env.GetPen().color(), Qt::SolidPattern ) );
				m_pPoint1 = pPoint;
			}	break;
			case 8:
			{
				QCanvasLine * pPoint1 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint2 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint3 = new QCanvasLine( env.GetCanvas() );
				pPoint1->setPoints( -5,-5,0,5 );
				pPoint1->setPen( env.GetPen() );
				pPoint1->setBrush( env.GetPen().color() );
				pPoint2->setPoints( 0,5,5,-5 );
				pPoint2->setPen( env.GetPen() );
				pPoint2->setBrush( env.GetPen().color() );
				pPoint3->setPoints( -5,-5,5,-5 );
				pPoint3->setPen( env.GetPen() );
				pPoint3->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint1;
				m_pPoint2 = pPoint2;
				m_pPoint3 = pPoint3;
			}	break;
			case 9:
			{
				QCanvasLine * pPoint1 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint2 = new QCanvasLine( env.GetCanvas() );
				QCanvasLine * pPoint3 = new QCanvasLine( env.GetCanvas() );
				pPoint1->setPoints( -5,5,0,-5 );
				pPoint1->setPen( env.GetPen() );
				pPoint1->setBrush( env.GetPen().color() );
				pPoint2->setPoints( 0,-5,5,5 );
				pPoint2->setPen( env.GetPen() );
				pPoint2->setBrush( env.GetPen().color() );
				pPoint3->setPoints( -5,5,5,5 );
				pPoint3->setPen( env.GetPen() );
				pPoint3->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint1;
				m_pPoint2 = pPoint2;
				m_pPoint3 = pPoint3;
			}	break;
			case 10:
			case 11:
			case 12:
			default:
			{
				QCanvasEllipse * pPoint = new QCanvasEllipse( env.GetCanvas() );
				pPoint->setSize( env.RelScaleX( POINT_X ), env.RelScaleY( POINT_Y ) );
				pPoint->setPen( env.GetPen() );
				pPoint->setBrush( env.GetPen().color() );
				m_pPoint1 = pPoint;
			}
		}
	}

	m_pPoint1->move( GetX( env ), GetY( env ) );
	m_pPoint1->show();

	if( m_pPoint2 )
	{
		m_pPoint2->move( GetX( env ), GetY( env ) );
		m_pPoint2->show();
	}
	if( m_pPoint3 )
	{
		m_pPoint3->move( GetX( env ), GetY( env ) );
		m_pPoint3->show();
	}
	if( m_pPoint4 )
	{
		m_pPoint4->move( GetX( env ), GetY( env ) );
		m_pPoint4->show();
	}

	// save the actual position for the next plot step...
	GrOpMove::Draw( env );
}

void GrOpPoint::Clear( GraphicEnv & env )
{
	delete m_pPoint1;
	m_pPoint1 = 0;
	delete m_pPoint2;
	m_pPoint2 = 0;
	delete m_pPoint3;
	m_pPoint3 = 0;
	delete m_pPoint4;
	m_pPoint4 = 0;
}

// ************************************************************************

GrOpVector::GrOpVector( int x, int y )
: GrOpMove( x, y ),
  m_pLine( 0 )
{
}

GrOpVector::~GrOpVector()
{
	delete m_pLine;
}

void GrOpVector::Draw( GraphicEnv & env )
{
	if( !m_pLine )
	{
		m_pLine = new QCanvasLine( env.GetCanvas() );
	}

	int xOld, yOld;
	env.GetPos( xOld, yOld );

	m_pLine->setPoints( env.ScaleX( xOld ), env.ScaleY( yOld ), GetX( env ), GetY( env ) );
	m_pLine->setPen( env.GetPen() );
	m_pLine->show();

	// save the actual position for the next plot step...
	GrOpMove::Draw( env );
}

void GrOpVector::Clear( GraphicEnv & env )
{
	delete m_pLine;
	m_pLine = 0;
}

// ************************************************************************

GrOpLinetype::GrOpLinetype( int iLinetype )
{
	m_iLinetype = iLinetype;
}

void GrOpLinetype::Draw( GraphicEnv & env )
{
	QBrush aActBrush = env.GetBrush();
	QPen aActPen = env.GetPen();

//	aActBrush.setColor( QColor( 0, 0, 0 ) );
//	aActBrush.setStyle( QBrush::NoBrush );
//	env.SetBrush( aActBrush );

	switch( m_iLinetype )
	{
		case -3:
			aActPen.setColor( QColor( 0, 0, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::DashLine );
			env.SetPen( aActPen );
			break;
		case -2:
			// border of the plot
			aActPen.setColor( QColor( 0, 0, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case -1:
			// for the x and y axis
			aActPen.setColor( QColor( 0, 0, 255 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 0:
			aActPen.setColor( QColor( 255, 0, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 1:
			aActPen.setColor( QColor( 0, 255, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 2:
			aActPen.setColor( QColor( 0, 0, 255 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 3:
			aActPen.setColor( QColor( 255, 255, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 4:
			aActPen.setColor( QColor( 255, 0, 255 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 5:
			aActPen.setColor( QColor( 0, 255, 255 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 6:
			aActPen.setColor( QColor( 128, 0, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 7:
			aActPen.setColor( QColor( 0, 128, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 8:
			aActPen.setColor( QColor( 0, 0, 128 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 9:
			aActPen.setColor( QColor( 128, 128, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		default: 
			;
	}
}

// ************************************************************************

GrOpPutText::GrOpPutText( int x, int y, const char * sText )
: GrOpMove( x, y ),
  m_pText( 0 )
{
	m_sText = sText;
}

GrOpPutText::~GrOpPutText()
{
	delete m_pText;
}

void GrOpPutText::Draw( GraphicEnv & env )
{
	if( !m_pText )
	{
		m_pText = new QCanvasText( m_sText, env.GetCanvas() );
	}

	m_pText->move( GetX( env )/*-m_pText->boundingRect().width()/2*/, GetY( env )-m_pText->boundingRect().height()/2 );
	m_pText->setColor( env.GetPen().color() );
	m_pText->show();
}

void GrOpPutText::Clear( GraphicEnv & env )
{
	delete m_pText;
	m_pText = 0;
}
