/**************************************************************************
 *
 *	project				 : qtplot
 *
 *	copyright            : (C) 2004 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source: $
 *
 *  $Revision: $
 *
 *	$Log: $
 *	
 *
 ***************************************************************************/
/***************************************************************************
 *																		   *
 * This file is part of the qtplot package (a gnuplot plotting frontend    *
 * for Qtopia and Qt)                                                      *
 *																		   *
 * Copyright (C) 2004 by Michael Neuroth.								   *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation; either version 2 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 ***************************************************************************/

// TODO --> arrow

#ifndef _GRAPHOP_H
#define _GRAPHOP_H

class QCanvas;
class QCanvasLine;
class QCanvasEllipse;
class QCanvasItem;
class QCanvasText;

#include <qstring.h>
#include <qbrush.h>
#include <qpen.h>

// size of the (virual) drawing area
#define MAX_QTOPIA_WIDTH	(4096+400)		// + 2 * offset
#define MAX_QTOPIA_HEIGHT	(4096+400)		// + 2 * offset
#define QTOPIA_X_OFFSET		200
#define QTOPIA_Y_OFFSET		200

// size of a (measurement) point in the (virtual) drawing area
#define POINT_X				50
#define POINT_Y				50

// ************************************************************************
// the environment for the graphic operations
class GraphicEnv
{
public:
	GraphicEnv( QCanvas * pCanvas );

	QCanvas *	GetCanvas() const;
	void		ClearCanvas();
	bool		IsCanvasOk() const;

	void		SetBrush( const QBrush & aBrush );
	QBrush		GetBrush() const;

	void		SetPen( const QPen & aPen );
	QPen		GetPen() const;

	void		SetPos( int x, int y );
	void		GetPos( int & x, int & y ) const;

	// for scaling absolute positions
	int			ScaleX( int x ) const;
	int			ScaleY( int y ) const;

	// for scaling lengths
	int			RelScaleX( int x ) const;
	int			RelScaleY( int y ) const;

private:
	QCanvas *	m_pCanvas;		// not an owner !
	QBrush		m_aActBrush;
	QPen		m_aActPen;
	
	int			m_x;
	int			m_y;
};

// ************************************************************************
// base class for graphic operations
class GraphicOperation
{
public:
	virtual ~GraphicOperation();

	virtual void Draw( GraphicEnv & env );
	virtual void Clear( GraphicEnv & env );
};

// ************************************************************************
// process the optioins
class GrOpOptions : public GraphicOperation
{
};

// ************************************************************************
// init the program
class GrOpInit : public GraphicOperation
{
};

// ************************************************************************
// exit the program
class GrOpReset : public GraphicOperation
{
};

// ************************************************************************
// switch to graphic modus
class GrOpGraphics : public GraphicOperation
{
};

// ************************************************************************
// switch to text mouds (back)
class GrOpText : public GraphicOperation
{
};

// ************************************************************************
// move to a point
class GrOpMove : public GraphicOperation
{
public:
	GrOpMove( int x, int y );

	virtual void Draw( GraphicEnv & env );

	int GetX( GraphicEnv & env ) const;
	int GetY( GraphicEnv & env ) const;

private:
	int		m_x;
	int		m_y;
};

// ************************************************************************
// draw a point
class GrOpPoint : public GrOpMove
{
public:
	GrOpPoint( int x, int y, int iPoint );
	virtual ~GrOpPoint();

	virtual void Draw( GraphicEnv & env );
	virtual void Clear( GraphicEnv & env );

private:
	int					m_iPoint;
	QCanvasItem *		m_pPoint1;
	QCanvasItem *		m_pPoint2;
	QCanvasItem *		m_pPoint3;
	QCanvasItem *		m_pPoint4;
};

// ************************************************************************
// draw a line
class GrOpVector : public GrOpMove
{
public:
	GrOpVector( int x, int y );
	virtual ~GrOpVector();

	virtual void Draw( GraphicEnv & env );
	virtual void Clear( GraphicEnv & env );

private:
	QCanvasLine *	m_pLine;
};

// ************************************************************************
// change the linetype
class GrOpLinetype : public GraphicOperation
{
public:
	GrOpLinetype( int iLinetype );

	virtual void Draw( GraphicEnv & env );

private:
	int		m_iLinetype;
};

// ************************************************************************
// draw a text
class GrOpPutText : public GrOpMove
{
public:
	GrOpPutText( int x, int y, const char * sText );
	virtual ~GrOpPutText();

	virtual void Draw( GraphicEnv & env );
	virtual void Clear( GraphicEnv & env );

private:
	QCanvasText *	m_pText;
	QString			m_sText;
};

#endif
