/**************************************************************************
 *
 *	project				 : qtplot
 *
 *	copyright            : (C) 2004 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source: $
 *
 *  $Revision: $
 *
 *	$Log: $
 *	
 *
 ***************************************************************************/
/***************************************************************************
 *																		   *
 * This file is part of the qtplot package (a gnuplot plotting frontend    *
 * for Qtopia and Qt)                                                      *
 *																		   *
 * Copyright (C) 2004 by Michael Neuroth.								   *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation; either version 2 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 ***************************************************************************/

#include "sockcomm.h"

ClientSocket::ClientSocket( int sock, QObject *parent, const char *name )
: QSocket( parent, name )
#ifdef SOCKET_BUG
  , m_pBuffer( 0 ), m_iBufferLen( 0 )
#endif
{
	connect( this, SIGNAL(readyRead()), SLOT(readClient()) );
	connect( this, SIGNAL(connectionClosed()), SLOT(deleteLater()) );
	setSocket( sock );
}

ClientSocket::~ClientSocket()
{
#ifdef SOCKET_BUG
	delete [] m_pBuffer;
#endif

	emit sigClosedSocket();
}

void ClientSocket::readClient()
{
#ifdef SOCKET_BUG
	unsigned long ulLen = bytesAvailable();

	char * pBuffer = new char [ulLen];

	unsigned long ulRead = readBlock( pBuffer, ulLen );
	
	if( ulRead>=0 )
	{
		addToCache( pBuffer, ulRead );

		while( isLineInCacheAvailable() )
		{
			QString str = removeFirstLineFromCache();

			emit sigReceivedText( str );
		}
	}

	delete [] pBuffer;

#else
	QTextStream ts( this );

	while ( canReadLine() ) 
	{
		QString str = ts.readLine();
		emit sigReceivedText( str );
		//emit sigReceivedText( tr("Read: '%1'\n").arg(str) );

		//ts << line << ": " << str << endl;
		//emit sigReceivedText( tr("Wrote: '%1: %2'\n").arg(line).arg(str) );

	}
#endif
}

#ifdef SOCKET_BUG

void ClientSocket::addToCache( char * pReadBuffer, int iLength )
{
	// create a new buffer to add the new data
	char * pNewBuffer = new char [m_iBufferLen+iLength];

	memcpy( pNewBuffer, m_pBuffer, m_iBufferLen );
	memcpy( pNewBuffer+m_iBufferLen, pReadBuffer, iLength );

	// now delete the old buffer
	delete [] m_pBuffer;

	m_pBuffer = pNewBuffer;
	m_iBufferLen = m_iBufferLen+iLength;
}

bool ClientSocket::isLineInCacheAvailable() const
{
	return getFirstCRPosInCache()>=0;
}

int ClientSocket::getFirstCRPosInCache() const
{
	for( int i=0; i<m_iBufferLen; i++ )
	{
		if( m_pBuffer[i]=='\n' )
		{
			return i;
		}
	}
	return -1;
}

QString ClientSocket::removeFirstLineFromCache()
{
	QString sRet;

	int iCRPos = getFirstCRPosInCache();
	if( iCRPos>=0 )
	{
		// copy the received bytes into the string
		for( int i=0; i<iCRPos; i++ )
		{
			sRet += m_pBuffer[i];
		}

		// resize the buffer (remove the returnd line from the buffer)
		{
			int iNewBufferLen = m_iBufferLen-(iCRPos+1);
			char * pNewBuffer = new char [iNewBufferLen];

			memcpy( pNewBuffer, m_pBuffer+iCRPos+1, iNewBufferLen );

			// now delete the old buffer
			delete [] m_pBuffer;

			m_pBuffer = pNewBuffer;
			m_iBufferLen = iNewBufferLen;
		}
	}

	return sRet;
}

#endif
