/**************************************************************************
 *
 *	project				 : qtplot
 *
 *	copyright            : (C) 2004 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source: $
 *
 *  $Revision: $
 *
 *	$Log: $
 *	
 *
 ***************************************************************************/
/***************************************************************************
 *																		   *
 * This file is part of the qtplot package (a gnuplot plotting frontend    *
 * for Qtopia and Qt)                                                      *
 *																		   *
 * Copyright (C) 2004 by Michael Neuroth.								   *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation; either version 2 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 ***************************************************************************/

#ifndef _SOCKCOMM_H
#define _SOCKCOMM_H

#include <qsocket.h>
#include <qserversocket.h>
#include <qtextstream.h>

#ifdef ZAURUS
#define SOCKET_BUG
#endif

/*
  The ClientSocket class provides a socket that is connected with a client.
  For every client that connects to the server, the server creates a new
  instance of this class.
*/
class ClientSocket : public QSocket
{
    Q_OBJECT
public:
    ClientSocket( int sock, QObject *parent=0, const char *name=0 );
    ~ClientSocket();

signals:
    void sigReceivedText( const QString& );
	void sigClosedSocket();

private slots:
    void readClient();

private:
#ifdef SOCKET_BUG
	void addToCache( char * pBuffer, int iLength );
	bool isLineInCacheAvailable() const;
	int getFirstCRPosInCache() const;
	QString removeFirstLineFromCache();

	char *		m_pBuffer;
	int			m_iBufferLen;
#endif
};

/*
  The SimpleServer class handles new connections to the server. For every
  client that connects, it creates a new ClientSocket -- that instance is now
  responsible for the communication with that client.
*/
class ServerSocket : public QServerSocket
{
    Q_OBJECT
public:
    ServerSocket( int port, QObject* parent=0 )
	: QServerSocket( (Q_UINT16)port, 1, parent )
    {
	/*	if ( !ok() ) 
		{
		    qWarning("Failed to bind to port 4242");
			exit(1);
		}
	*/
    }

    ~ServerSocket()
    {
    }

    void newConnection( int socket )
    {
		ClientSocket *s = new ClientSocket( socket, this );
		emit sigNewConnect( s );
    }

signals:
    void sigNewConnect( ClientSocket* );
};

#endif
